Package: gretel
Title: Generalized Path Analysis for Social Networks
Version: 0.0.1
Date: 2019-08-09
Authors@R: person("David", "Buch", email = "davidbuch42@gmail.com", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4574-0075"))
Description: The social network literature features numerous methods for assigning
    value to paths as a function of their ties. 'gretel' systemizes these approaches,
    casting them as instances of a generalized path value function indexed by 
    a penalty parameter. The package also calculates probabilistic path value and
    identifies optimal paths in either value framework. Finally, proximity 
    matrices can be generated in these frameworks that capture high-order connections 
    overlooked in primitive adjacency sociomatrices. Novel methods are described
    in Buch (2019) <https://davidbuch.github.io/analyzing-networks-with-gretel.html>. 
    More traditional methods are also implemented, as described in Yang, Knoke (2001) 
    <doi:10.1016/S0378-8733(01)00043-0>.
Maintainer: David Buch <davidbuch42@gmail.com>
URL: https://github.com/davidbuch/gretel
BugReports: https://github.com/davidbuch/gretel/issues
License: GPL-3
Depends: R (>= 3.0)
Imports: Rcpp (>= 1.0.0), ResistorArray (>= 1.0-32)
LinkingTo: Rcpp
Suggests: knitr, rmarkdown, testthat (>= 2.1.0)
VignetteBuilder: knitr
LazyData: true
RoxygenNote: 6.1.1
NeedsCompilation: yes
Packaged: 2019-08-21 11:38:45 UTC; Buch
Author: David Buch [aut, cre] (<https://orcid.org/0000-0002-4574-0075>)
Repository: CRAN
Date/Publication: 2019-08-22 12:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 09:52:03 UTC; unix
Archs: gretel.so.dSYM

# Generated by roxygen2: do not edit by hand

export(all_opt_gpv)
export(all_opt_ppv)
export(binary_distance)
export(flament_average_path_length)
export(flament_path_length)
export(generate_proximities)
export(gpv)
export(opt_gpv)
export(opt_ppv)
export(peay_average_path_value)
export(peay_path_value)
export(ppv)
export(unpack)
importFrom(Rcpp,sourceCpp)
useDynLib(gretel, .registration = TRUE)

<div align="center">
<p align="center">
    <img src="gretel-logo.png?raw=true?" alt="gretel-logo" width="200">
</p>
<h1 align="center">Gretel</h1>
<h3 align="center">An algorithm for recovering haplotypes from metagenomes. Sister to <a href="https://github.com/SamStudio8/hansel">Hansel</a>.
</h3>
<p align="center">
<a href="https://github.com/samstudio8/gretel/blob/master/LICENSE"><img src="https://img.shields.io/badge/license-MIT-orange.svg" alt="License"></a>
<a href="https://bioconda.github.io/recipes/gretel/README.html"><img src="https://anaconda.org/bioconda/gretel/badges/downloads.svg" alt="bioconda"></a>
</p>
</div>


What is it?
-----------

**Gretel** is a Python package providing a command line tool for the recovery of haplotypes
from metagenomic data sets. **Gretel** parses an alignment of reads into a **Hansel** matrix
and uses the evidence of SNP pairs observed to appear on the same reads to probabilistically
reconstruct the most likely haplotypes.

**Gretel** uses an L'th order Markov chain model to reconstruct likely sequences
of variants that constitute haplotypes in the real metagenome.
Our approach involves graph-like traversal of the data within the **Hansel** matrix.
Edges are probabilitically weighted based on the evidence on the reads, as well as
the haplotype as it has been reconstructed so far.

What can I use it for?
----------------------

**Gretel** is designed to recover haplotypes from your data set, without the need for
setting (or optimisation) of any parameters.
**Gretel** does not require a priori knowledge of your input data (such as its contents, or
the true number of haplotypes) and makes no assumptions
regarding the distributions of alleles at variant sites and uses the available evidence
from the aligned reads without altering or discarding the observed varations.

Why should I use it?
--------------------

**Gretel** is the first tool capable of recovering haplotypes from metagenomes.
Whilst tools exist for analogous haplotyping problems, such as the assembly of
viral quasispecies, typically these tools rely on overlap approaches that create
too many unranked haplotypes. **Gretel** is capable of ranking the haplotypes it
outputs by their likelihood.

**Gretel** requires no parameters and our approach is robust to sequencing error
and misalignment noise.

Requirements
------------


    $ pip install numpy hanselx pysam PyVCF

Install
-------


    $ pip install gretel

Alternatively, Gretel has been packaged for bioconda (Thanks [@johnne!](https://github.com/johnne)):

    $ conda install -c bioconda gretel

Usage
-----
You will require a sorted BAM containing your reads, aligned to some pseudo-reference.
You can use any sequence as your reference, such as a consensus assembly of the
metagenomic reads, or a known strain reference (such as HIV-1).
You must bgzip and tabix your VCF.

    $ gretel <bam> <vcf.gz> <contig> -s <1-start> -e <1-end> --master <master.fa> -o <outdir>


Citation (Pre-Print)
--------
```
@article {Nicholls223404,
        author = {Nicholls, Samuel M. and Aubrey, Wayne and Edwards, Arwyn and Grave, Kurt de and Huws, Sharon and Schietgat, Leander and Soares, Andr{\'e} and Creevey, Christopher J. and Clare, Amanda},
        title = {Recovery of gene haplotypes from a metagenome},
        elocation-id = {223404},
        year = {2019},
        doi = {10.1101/223404},
        publisher = {Cold Spring Harbor Laboratory},
        URL = {https://www.biorxiv.org/content/early/2019/10/17/223404},
        eprint = {https://www.biorxiv.org/content/early/2019/10/17/223404.full.pdf},
        journal = {bioRxiv}
}
```

License
-------
Hansel and Gretel are distributed under the MIT license, see LICENSE.

## ---- include = FALSE----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup---------------------------------------------------------------
library(gretel)
BuchDarrah19

## ------------------------------------------------------------------------
opt_gpv(BuchDarrah19, source = 1, target = 5, p = 0)

## ------------------------------------------------------------------------
opt_gpv(BuchDarrah19, source = 1, target = 5, p = 1)

## ------------------------------------------------------------------------
opt_gpv(BuchDarrah19, source = 1, target = 5, p = Inf)

## ------------------------------------------------------------------------
optimal_path <- opt_ppv(BuchDarrah19, source = 1, target = 5, odds_scale = 4)
print(optimal_path)

## ------------------------------------------------------------------------
prob_path_value <- ppv(BuchDarrah19, path = optimal_path, odds_scale = 4)
transmission_odds <- prob_path_value/4
print(prob_path_value)
print(transmission_odds)

## ------------------------------------------------------------------------
generate_proximities(BuchDarrah19, mode = "ogpv", p = 1)

generate_proximities(BuchDarrah19, mode = "oppv", odds_scale = 4)

generate_proximities(BuchDarrah19, mode = "sconductivity")

## ------------------------------------------------------------------------
# Suppose we wish to calculate the betweenness centrality of node 3 in 
# the example sociomatrix 'BuchDarrah19'

# There are n-1 choose 2 pairs of nodes for which neither the source nor the target is 3
# Since n = 5 in this case, there are 6 pairs of nodes to consider.
# Therefore, node 3 mediates at most 6 shortest paths.
# (If this were a directed graph, this number would be 12)

all_paths <- all_opt_gpv(BuchDarrah19, p = 1)
paths_mediated <- 0
# We can get away with just looking at half the sociomatrix (below) because this
# one is undirected.
for(i in 1:5){
  for(j in (i+1:5)){
    shortest_ij <- unpack(all_paths[[i]], i, j)
    if(3 %in% shortest_ij) paths_mediated <- paths_mediated + 1
  }
}

# Print Betweenness Centrality
print(paths_mediated/6)


## What about if p = 0?
all_paths <- all_opt_gpv(BuchDarrah19, p = 0)
paths_mediated <- 0
# We can get away with just looking at half the sociomatrix (below) because this
# one is undirected.
for(i in 1:5){
  for(j in (i+1:5)){
    shortest_ij <- unpack(all_paths[[i]], i, j)
    if(3 %in% shortest_ij) paths_mediated <- paths_mediated + 1
  }
}

# Print Betweenness Centrality
print(paths_mediated/6)

#' sconduct: Generalized Path Analysis for Social Networks
#' 
#' This package contains two categories of functions. The first category is concerned
#' with assigning values to user specified paths, while the second identifies
#' paths of optimal value.
#' 
#' Key functions in the path value calculation category are
#' - \code{gpv}, which calculates Generalized Path Value
#' - \code{ppv}, which calculates Probabilistic Path Value
#' - \code{binary_distance}, \code{peay_path_value}, \code{flament_path_length},
#'   \code{peay_average_path_value}, and \code{flament_average_path_length}, which
#'   calculate path value measures described in \emph{Yang, Knoke} (2001).
#' - \code{generate_proximities}, which generates a matrix of values representing the
#'   measures of optimal paths from each source node (row index) to each target node 
#'   (column index).
#'   
#' Key functions in the optimal path identification category are
#' - \code{opt_gpv}, which identifies the path of optimal Generalized Path Value from 
#'   a particular source node to a particular target node
#' - \code{opt_ppv}, which identifies the path of optimal Probabilistic Path Value from
#'   a particular source node to a particular target node
#' - \code{all_opt_gpv}, which identifies the 'gpv'-optimal paths from every source node 
#'   to every target node
#' - \code{all_opt_ppv}, which identifies the 'ppv'-optimal paths from every source node
#'   to every target node
#' - \code{unpack}, which unpacks the Dijkstra-format encoded shortest paths returned by
#'   \code{all_opt_gpv} and \code{all_opt_ppv}. See their help pages for details.
#'
#' @docType package
#' @name gretel
NULL


## usethis namespace: start
#' @useDynLib gretel, .registration = TRUE
## usethis namespace: end
NULL

## usethis namespace: start
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
NULL

as_binary <- function(sociomatrix){
  return(matrix(as.numeric(as.logical(sociomatrix)),nrow = nrow(sociomatrix)))
}

sum_path <- function(edge_data, path){
  path_sum <- 0
  nsteps <- length(path) - 1
  for(i in 1:nsteps){
    on <- path[i]
    to <- path[i + 1]
    path_sum <- path_sum + edge_data[on, to]
  }
  return(path_sum)
}

#' Unpacks a Path from a Dijkstra-Format Spanning Tree
#'
#' Used with \code{all_opt_gpv} and \code{all_opt_ppv} to 
#' unpack individual paths from the Dijkstra-format trees that 
#' those functions return.
#' 
#' Returns \code{NA} if a path does not exist
#' 
#' @param tree a Dijkstra-format tree returned by \code{all_opt_gpv} or \code{all_opt_ppv}
#' @param source an integer index corresponding to a node in \code{sociomatrix}
#' @param target an integer index corresponding to a node in \code{sociomatrix}
#' 
#' @example
#' # Identify the optimal paths
#' best_paths <- all_opt_gpv(YangKnoke01, p = 1)
#' 
#' # 'best_paths' will contain a list of trees in Dijkstra's format.
#' # 'best_paths[[i]]' is the tree encoding shortest paths from source
#' #     node 'i' to all alters. We can return the optimal path from 
#' #     node 1 to node 4 as follows.
#' unpack(best_paths[[1]], source = 1, target = 4)
#' 
#' @export
unpack<- function(tree, source, target){
  # maybe just accept unpacking the path backwards to save memory reallocations
  if(is.na(tree[target])){
    path <- NA
  }else{
    on <- target
    path <- c(on)
    while(on != source){
      prev <- tree[on]
      path <- c(prev,path)
      on <- prev
    }
  }
  return(path) 
}


min_tie <- function(sociomatrix, path){
  nties <- length(path) - 1
  tie_vals <- c()
  for(i in 1:nties){
    on <- path[i]
    to <- path[i + 1]
    tie_vals <- c(tie_vals, sociomatrix[on, to])
  }
  return(min(tie_vals))
}

scale_to_odds <- function(sociomatrix, odds_scale, odds_scale_by_node){
  if(is.null(odds_scale_by_node)){
    sociomatrix <- sociomatrix/odds_scale
  }else{
    nv <- nrow(sociomatrix)
    for(i in 1:nv){
      sociomatrix[i,] <- sociomatrix[i,]/odds_scale_by_node[i]
    }
  }
  return(sociomatrix)
}

shortest_path <- function(distance_matrix, source, target, node_costs, p_finite = TRUE){
  if(p_finite == TRUE){
    prev <- dijkstra_nodes(distance_matrix, source, node_costs)
  } else {
    prev <- dijkstra_inf(distance_matrix, source)
  }
  if(is.na(prev[target])){
    path <- NA # path is disconnected
  } else {
    # Now walk backwards through 'prev'
    on <- target
    path <- c(on)
    while(on != source){
      on <- prev[on]
      path <- c(on, path)
    }
  }
  return(path)
}

APSP <- function(dist, node_costs, p_finite = TRUE){
  nv <- nrow(dist)
  paths <- list()
  if(p_finite == TRUE){
    for(s in 1:nv){
      paths[[s]] <- dijkstra_nodes(dist = dist, src = s, node_costs = node_costs)
    }
  }else{
    for(s in 1:nv){
      paths[[s]] <- dijkstra_inf(dist = dist, src = s)
    }
  }
  return(paths)
}

sconduct <- function(sociomatrix){
  nv <- nrow(sociomatrix)
  for(i in 1:nv){sociomatrix[i,i] <- 0}
  laplacian <- diag(apply(sociomatrix,MARGIN = 1,sum)) - sociomatrix
  sconductivity <- 1/ResistorArray::Wu(laplacian)
  return(sconductivity)
}

check_input <- function(sociomatrix, path=1, source=1, target=1, p_norm=1, node_costs = NULL, odds_scale=1, odds_scale_by_node = NULL){
  # Check sociomatrix
  if(!is.matrix(sociomatrix)){
    stop("'sociomatrix' must be class 'matrix'")
  }
  if(!is.numeric(sociomatrix)){
    stop("'sociomatrix' must be type 'numeric'")
  }
  if(nrow(sociomatrix) != ncol(sociomatrix)){
    stop("'sociomatrix' must be square")
  }
  actors <- 1:ncol(sociomatrix) # used in checking path
  
  # Check path
  if(is.null(path)){
   stop("'path' of node indices must be provided") 
  }
  if(any(!(path %in% actors))){
    stop("Not all node indices in 'path' correspond to indices in 'sociomatrix'")
  }
  
  # Check source and target
  if(is.null(source) || is.null(target)){
    stop("A source and target vertex must be provided")
  }
  if((length(source) != 1) || (length(target) != 1)){
    stop("'source' and 'target' must be unique")
  }
  if(!((source %in% actors) && (target %in% actors))){
    stop("'source' and 'target' must both correspond to vertex indices in 'sociomatrix'")
  }
  
  # Check p_norm
  if(is.null(p_norm)){
    stop("'p_norm' must be specified")
  }
  if(!(is.numeric(p_norm) && (length(p_norm) == 1))){
    stop("'p_norm' must be a unique numeric")
  }
  if(p_norm < 0){
    stop("'p_norm' must be nonnegative")
  }
  
  #check odds_scale
  if(is.null(odds_scale)){
    stop("'odds_scale' must be specified")
  }
  if(!(is.numeric(odds_scale) && (length(odds_scale) == 1))){
    stop("'odds_scale' must be a unique numeric")
  }
  if(odds_scale <= 0){
    stop("'odds_scale' must be positive")
  }
  
  # check node_costs, if provided
  if(!is.null(node_costs)){
    if(!is.numeric(node_costs) || length(node_costs) != length(actors)){
      stop("'node_costs' must be a numeric vector of length 'nrow(sociomatrix)'")
    }
    if(any(node_costs < 0)){
      stop("'node_costs' must be nonnegative")
    }
  }
  
  # check odds_scale_by_node, if provided
  if(!is.null(odds_scale_by_node)){
    if(!is.numeric(odds_scale_by_node) || length(odds_scale_by_node) != length(actors)){
      stop("'odds_scale_by_node' must be a numeric vector of length 'nrow(sociomatrix)'")
    }
    if(any(odds_scale_by_node <= 0)){
      stop("'odds_scale_by_node' values must be positive")
    }
  }
}

#' Calculate probabilistic path value
#'
#' Given a real valued sociomatrix, a path, and an optional \code{odds_scale}, \code{ppv}
#' calculates the transmission odds for the path and returns the transmission odds
#' times \code{odds_scale} so the result can be directly compared with observed 
#' tie strenghts.
#'
#' We assume that observed tie strengths in \code{sociomatrix} are linearly
#' proportional to transmission odds. That is, if the transmission odds for
#' a strength 1 tie are 1 to 1, the transmission odds for a strength 5 tie are
#' 1 to 5.
#'
#'
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param path an integer vector of node indices from \code{sociomatrix}.
#' @param odds_scale a nonnegative real number indicating the observed tie
#'     strength value that corresponds to 1-1 transmission odds
#' @param odds_scale_by_node sets a transfer odds scale for each node in a 
#'     probabilistic path value calculation.  
#'     
#' @seealso \code{\link{opt_ppv}} to identify the path of optimal 'ppv' between two nodes 
#'     and \code{\link{all_opt_ppv}} to identify the optimal paths between all pairs of 
#'     nodes. Calling \code{\link{generate_proximities}} with \code{mode = 'ppv'} 
#'     returns a matrix 'ppv' values for the optimal paths between all pairs of 
#'     nodes.
#' 
#' @examples
#' ## Calculate ppv along a path in a sociomatrix
#' ppv(YangKnoke01, path = c(1,2,5), odds_scale = 3)
#' 
#' ## This path doesn't exist
#' gpv(YangKnoke01, path = c(1,2,4,5))
#' 
#' @export
ppv <- function(sociomatrix, path, odds_scale = 1, odds_scale_by_node = NULL){
  check_input(sociomatrix, path = path, odds_scale = odds_scale, odds_scale_by_node = odds_scale_by_node)
  odds_matrix <- scale_to_odds(sociomatrix, odds_scale, odds_scale_by_node)
  probability_matrix <- odds_matrix/(1+odds_matrix)
  lpm <- log(probability_matrix)
  log_prob <- sum_path(lpm, path)
  transmission_prob <- exp(log_prob)
  transmission_odds <- transmission_prob/(1 - transmission_prob)
  return(transmission_odds*odds_scale)
}

#' Generalized Path Value
#'
#' Calculates the generalized path value of a user-specified path through 
#' \code{sociomatrix}. Parameter \code{p} sets the p-norm used in calculation.
#' 
#' As a rule of thumb, p close to 0 will downweight the impact of particular
#' tie strengths and upweight the impact of binary path length. p equal to 
#' infinity will recapitulate the traditional path value measure of Peay (1980)
#' and is therefore the default. In other words, the value of a path under 
#' \code{p = Inf} will be the value of the weakest tie. The value of the same
#' path under \code{p = 0} will be the inverse of its binary length.
#' 
#'
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param path an integer vector of node indices from \code{sociomatrix}.
#' @param p a nonnegative real number that sets the 'p-norm' parameter for 
#'     generalized path value calculation.
#' @param node_costs a list of costs, in order, of all nodes represented in the 
#'     sociomatrix, all are assumed 0 if unspecified
#'     
#' @seealso \code{\link{opt_gpv}} to identify the path of optimal 'gpv' between two nodes 
#'     and \code{\link{all_opt_gpv}} to identify the optimal paths between all pairs of 
#'     nodes. Calling \code{\link{generate_proximities}} with \code{mode = 'gpv'} 
#'     returns a matrix 'gpv' values for the optimal paths between all pairs of 
#'     nodes.
#' 
#' @examples
#' ## Calculate gpv along a path in a sociomatrix
#' gpv(YangKnoke01, path = c(1,2,5), p = 1)
#' 
#' ## The same calculation, with nonzero node costs
#' gpv(YangKnoke01, path = c(1,2,5), p = 1, node_costs = c(1,3,3,2,1))
#' 
#' ## This path doesn't exist
#' gpv(YangKnoke01, path = c(1,2,4,5), p = 0)
#' 
#' @export
gpv <- function(sociomatrix, path, p = Inf, node_costs = NULL){
  check_input(sociomatrix, path = path, p_norm = p, node_costs = node_costs)
  if(is.null(node_costs)){
    node_costs <- rep(0, nrow(sociomatrix))
  }
  intermediate_nodes <- path[-c(1,length(path))]
  if(p == 0){
    # effective distance matrix
    d_eff_mat <- 1/as_binary(sociomatrix)
    d_eff_path <- sum_path(d_eff_mat, path) + sum(node_costs[intermediate_nodes])
    path_value <- 1/d_eff_path
  } else if(p < Inf){
    d_eff_mat <- 1/(sociomatrix^p)
    d_eff_path <- sum_path(d_eff_mat, path) + sum(node_costs[intermediate_nodes])
    path_value <- (1/d_eff_path)^(1/p)
  } else if(p == Inf){
    path_value <- min_tie(sociomatrix, path)
  }
  return(path_value)
}

#' Generate a Proximity Matrix
#' 
#' Generates a proximity matrix in one of three modes:
#' \describe{
#'   \item{\code{'ogpv'}}{Optimal Generalized Path Value. Entry \code{i,j} of the 
#'     proximity matrix will equal the optimal 'gpv' among all paths connecting node 
#'     \code{i} to node \code{j}.}
#'   \item{\code{'oppv'}}{Optimal Probabilistic Path Value. Entry \code{i,j} of the 
#'     proximity matrix will equal the optimal 'ppv' among all paths connecting node 
#'     \code{i} to node \code{j}.}
#'   \item{\code{'sconductivity'}}{Social Conductivity (Random Walk Probability). If 
#'     each tie strength recorded in \code{sociomatrix} is taken to be analogous to the 
#'     conductivity of an electrical component, \code{i,j} of the proximity matrix 
#'     will equal total conductivity of all paths from node \code{i} to node \code{j}.}
#' }
#' 
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param mode a selection of \code{'ogpv'}, \code{'oppv'}, or \code{'sconductivity'}
#' @param p if \code{mode} is \code{'ogpv'}, determines 'p-norm' parameter for 
#'     generalized path value calculation.
#' @param node_costs if \code{mode} is \code{'ogpv'}, assigns transmission costs to
#'     vertices within the sociomatrix.
#' @param odds_scale if \code{mode} is \code{'oppv'}, sets a global transfer odds 
#'     scale for probabilistic path value calculation.
#' @param odds_scale_by_node if \code{mode} is \code{'oppv'}, sets a transfer odds 
#'     scale for each node in a probabilistic path value calculation.
#' 
#' @examples
#' ## Generate a proximity matrix in each mode
#' ## Optimal Generalized Path Value
#' generate_proximities(YangKnoke01, mode = "ogpv", p = Inf, node_costs = c(1,3,3,2,1))
#' 
#' ## Optimal Probabilistic Path Value
#' generate_proximities(YangKnoke01, mode = "oppv", odds_scale = 2)
#' 
#' ## Sconductivity
#' generate_proximities(YangKnoke01, mode = "sconductivity")
#' 
#' @seealso \code{\link{gpv}}, \code{\link{ppv}}
#' @export
generate_proximities <- function(sociomatrix, mode = c("ogpv", "oppv", "sconductivity"), p = Inf, node_costs = NULL, odds_scale = 1, odds_scale_by_node = NULL){
  check_input(sociomatrix, p_norm = p, node_costs = node_costs, odds_scale = odds_scale, odds_scale_by_node = odds_scale_by_node)
  mode <- mode[1] # "ogpv" is default if no mode is specified.
  if(!(mode %in% c("ogpv", "oppv", "sconductivity"))){
    stop("'mode' must be one of 'ogpv', 'oppv', or 'sconductivity'")
  }
  nv <- nrow(sociomatrix)
  if(mode == "ogpv"){
    if((!is.null(odds_scale_by_node)) || (odds_scale != 1)){warning("Odds scale parameters not used for 'mode = ogpv'")}
    trees <- all_opt_gpv(sociomatrix, p, node_costs)
    proximity_matrix <- matrix(0, nrow = nv, ncol = nv)
    for(i in 1:nv){
      tree <- trees[[i]]
      for(j in 1:nv){
        path <- unpack(tree,i,j)
        if(i == j){
          proximity_matrix[i,j] <- Inf
        }else if(anyNA(path)){
          proximity_matrix[i,j] <- 0
        }else{
          proximity_matrix[i,j] <- gpv(sociomatrix, path, p, node_costs)
        }
      }
    }
  }else if(mode == "oppv"){
    if((!is.null(node_costs)) || (p != Inf)){warning("Node cost parameter and p norm parameter not used for 'mode = oppv'")}
    trees <- all_opt_ppv(sociomatrix, odds_scale, odds_scale_by_node)
    proximity_matrix <- matrix(0, nrow = nv, ncol = nv)
    for(i in 1:nv){
      tree <- trees[[i]]
      for(j in 1:nv){
        path <- unpack(tree,i,j)
        if(i == j){
          proximity_matrix[i,j] <- Inf
        }else if(anyNA(path)){
          proximity_matrix[i,j] <- 0
        }else{
          proximity_matrix[i,j] <- ppv(sociomatrix, path, odds_scale, odds_scale_by_node)
        }
      }
    }
  }else if(mode == "sconductivity"){
    if((!is.null(node_costs)) || (p != Inf) || (!is.null(odds_scale_by_node)) || (odds_scale != 1)){
      warning("Odds scale, node cost, and p norm parameters not used for 'mode = sconductivity'")
    }
    if(!isSymmetric(sociomatrix)){
      warning("sconductivity beta - intended only for undirected networks and no node costs")
    }
    proximity_matrix <- sconduct(sociomatrix)
  }
  return(proximity_matrix)
}

context("Proximity generation")
library(gretel)

test_that("Mode 'ogpv' works", {
  # check for node costs and no node costs
  # expect_equal_to_reference(generate_proximities(BuchDarrah19, mode = "ogpv", p = 3, node_costs = c(2,3,2,1,0)), file = "tests/testthat/ogpv_prox_cache")
})

test_that("Mode 'oppv' works", {
  # check for odds scale = 2 and odds_scale by node
  # expect_equal_to_reference(generate_proximities(BuchDarrah19, mode = "oppv", odds_scale_by_node = c(1,2,3,2,1)), file = "tests/testthat/oppv_prox_cache")
})

test_that("Mode 'sconduct' works", {
  # expect_equal_to_reference(generate_proximities(BuchDarrah19, mode = "sconductivity"), file = "tests/testthat/scond_prox_cache")
})

#' Binary Distance of a Network Path
#' 
#' Calculates the binary distance of a user-specified network path through a network,
#' if all edges exist. Otherwise, returns \code{Inf} to signify infinite distance.
#'
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param path an integer vector of node indices from \code{sociomatrix}.
#' 
#' @examples
#' ## Calculate binary distance along a path in a sociomatrix
#' binary_distance(YangKnoke01, path = c(1,2,5))
#' 
#' ## This path doesn't exist
#' binary_distance(YangKnoke01, path = c(1,2,4,5))
#' 
#' @export
binary_distance <- function(sociomatrix, path){
  check_input(sociomatrix, path = path)
  return(1/gpv(sociomatrix, path, p = 0))
}

#' Peay's Path Value Measure
#'
#' Calculates path value as defined in Peay (1980). That is, returns the
#' value of the weakest connection in the path, if all edges exist. 
#' Otherwise, returns 0.
#'
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param path an integer vector of node indices from \code{sociomatrix}.
#' 
#' @examples
#' ## Calculate Peay's Path Value along a path in a sociomatrix
#' peay_path_value(YangKnoke01, path = c(1,2,5))
#' 
#' ## This path doesn't exist
#' peay_path_value(YangKnoke01, path = c(1,2,4,5))
#' 
#' @seealso \code{\link{peay_average_path_value}}
#' @export
peay_path_value <- function(sociomatrix, path){
  check_input(sociomatrix, path = path)
  return(gpv(sociomatrix, path, p = Inf))
}

#' Flament's Path Length Measure
#'
#' Calculates path length as defined in Flament (1963). That is, sums the
#' values of each edge in the path, if all edges exist. Otherwise, returns \code{NA}.
#'
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param path an integer vector of node indices from \code{sociomatrix}.
#'
#' @examples
#' ## Calculate Flament's Path Length along a path in a sociomatrix
#' flament_path_length(YangKnoke01, path = c(1,2,5))
#' 
#' ## This path doesn't exist
#' flament_path_length(YangKnoke01, path = c(1,2,4,5))
#'
#' @seealso \code{\link{flament_average_path_length}}
#' @export
flament_path_length <- function(sociomatrix, path){
  check_input(sociomatrix, path = path)
  path_sum <- 0
  nsteps <- length(path) - 1
  for(i in 1:nsteps){
    on <- path[i]
    to <- path[i + 1]
    if(sociomatrix[on,to] == 0){return(NA)}
    path_sum <- path_sum + sociomatrix[on, to]
  }
  return(path_sum)
}

#' Yang and Knoke's Average Path Value
#' 
#' Calculates 'APV' (Average Path Value) as defined in Yang, Knoke (2001)
#' Called \code{peay_average_path_value} in homage to E.R. Peay, who defined 
#' path length in 1980.
#' 
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param path an integer vector of node indices from \code{sociomatrix}.
#' 
#' @examples
#' ## Calculate 'APV' of a path in a sociomatrix
#' peay_average_path_value(YangKnoke01, path = c(1,2,5))
#' 
#' ## This path doesn't exist
#' peay_average_path_value(YangKnoke01, path = c(1,2,4,5))
#' 
#' @seealso \code{\link{peay_path_value}}
#' @export
peay_average_path_value <- function(sociomatrix, path){
  #peay_path_value() will handle input checking
  return(peay_path_value(sociomatrix, path)/(length(path) - 1))
}

#' Yang and Knoke's Average Path Length
#' 
#' Calculates 'APL' (Average Path Length) as defined in Yang, Knoke (2001).
#' Called \code{flament_average_path_length} in homage to A.C. Flament, who defined
#' path length in 1963.
#' 
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param path an integer vector of node indices from \code{sociomatrix}.
#' 
#' @examples
#' ## Calculate 'APL' of a path in a sociomatrix
#' flament_average_path_length(YangKnoke01, path = c(1,2,5))
#' 
#' ## This path doesn't exist
#' flament_average_path_length(YangKnoke01, path = c(1,2,4,5))
#' 
#' @seealso \code{\link{flament_path_length}}
#' @export
flament_average_path_length <- function(sociomatrix, path){
  #flament_path_length() will handle input checking
  return(flament_path_length(sociomatrix, path)/(length(path) - 1))
}

context("Path values")
library(gretel)


path1 <- c(1,2,3)
path2 <- c(1,5,2,3)
path3 <- c(1,5,4,3)
not_a_path <- c(1,5,3)
junk <- matrix(c(1,"a",3,3,2,1), nrow = 2)

test_that("Historical measures recover Yang, Knoke (2001)", {
  expect_equal(binary_distance(YangKnoke01, path1), 2)
  expect_equal(binary_distance(YangKnoke01, path2), 3)
  expect_equal(binary_distance(YangKnoke01, path3), 3)
  
  expect_equal(peay_path_value(YangKnoke01, path1), 1)
  expect_equal(peay_path_value(YangKnoke01, path2), 2)
  expect_equal(peay_path_value(YangKnoke01, path3), 3)
  
  expect_equal(flament_path_length(YangKnoke01, path1), 3)
  expect_equal(flament_path_length(YangKnoke01, path2), 8)
  expect_equal(flament_path_length(YangKnoke01, path3), 13)
  
  expect_equal(peay_average_path_value(YangKnoke01, path2), 2/3)
  expect_equal(flament_average_path_length(YangKnoke01, path2), 8/3)
})

test_that("Historical measures detect bad inputs", {
  # Not a real path
  # Not the right format
  expect_equal(binary_distance(YangKnoke01, path = not_a_path), Inf)
  expect_error(binary_distance(YangKnoke01, path = junk))
  
  expect_equal(peay_path_value(YangKnoke01, path = not_a_path), 0)
  expect_error(peay_path_value(YangKnoke01, path = junk))
  
  expect_equal(flament_path_length(YangKnoke01, path = not_a_path), NA)
  expect_error(flament_path_length(YangKnoke01, path = junk))
})

test_that("'gpv' and 'ppv' evaluate correctly", {
  #two values of p
  expect_equal(gpv(YangKnoke01, path = path1, p = Inf), 1)
  expect_equal(gpv(YangKnoke01, path = path1, p = 1), 2/3)
  #one value of p with node costs
  expect_equal(gpv(YangKnoke01, path = path1, p = 1, node_costs = c(1,2,1,2,1)), 0.2857143)
  #two values of odds_scale
  expect_equal(ppv(YangKnoke01, path = path1, odds_scale = 1), 1/2)
  expect_equal(ppv(YangKnoke01, path = path1, odds_scale = 2), 4/10)
  #odds scale by node
  expect_equal(ppv(YangKnoke01, path = path1, odds_scale_by_node = c(2,1,1,2,3)), 0.2857143)
})

test_that("'gpv' and 'ppv' detect bad inputs", {
  # Not the right format path
  expect_error(ppv(YangKnoke01, path = junk))
  # not the right format p
  expect_error(gpv(YangKnoke01, path = path1, p = -1))
  expect_error(gpv(YangKnoke01, path = path1, p = "a"))
  # not the right format node costs
  expect_error(gpv(YangKnoke01, path = path1, node_costs = c(-5,2,3,1,2)))
  # not the right format odds_scale
  expect_error(ppv(YangKnoke01, path = path1, odds_scale = "a"))
  # not the right format odds_scale_by_node
  expect_error(ppv(YangKnoke01, path = path1 , odds_scale_by_node = c(1,2)))
  # Not a real path
  expect_equal(gpv(YangKnoke01, path = not_a_path, p = 2), 0)
  expect_equal(ppv(YangKnoke01, path = not_a_path, odds_scale = 3), 0)
})

#' Optimize Probabilistic Path Value
#' 
#' Identify the path of optimal probabilistic path value from a source node
#' to a target node.
#' 
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param source an integer index corresponding to a node in \code{sociomatrix}
#' @param target an integer index corresponding to a node in \code{sociomatrix} 
#' @param odds_scale a nonnegative real number indicating the observed tie
#'     strength value that corresponds to 1-1 transmission odds
#' @param odds_scale_by_node sets a transfer odds scale for each node in a 
#'     probabilistic path value calculation.  
#' 
#' @seealso \code{\link{ppv}} to calculate the value of a user-specified path,
#'     \code{\link{all_opt_ppv}} to simultaneously identify the optimal paths 
#'     from any source node to any target node.
#' 
#' @example
#' # Identify the optimal path
#' best_path <- opt_ppv(YangKnoke01, source = 1, target = 4, odds_scale = 1)
#' 
#' # Print the gpv of the optimal path
#' ppv(YangKnoke01, path = best_path, odds_scale = 1)
#' 
#' # Print the gpv of an inferior path
#' ppv(YangKnoke01, path = c(1,2,3,4))
#' 
#' @export
opt_ppv <- function(sociomatrix, source, target, odds_scale = 1, odds_scale_by_node = NULL){
  check_input(sociomatrix, source = source, target = target, odds_scale = odds_scale, odds_scale_by_node = odds_scale_by_node)
  odds_matrix <- scale_to_odds(sociomatrix, odds_scale, odds_scale_by_node)
  probability_matrix <- odds_matrix/(1+odds_matrix)
  lpm <- log(probability_matrix)
  d_eff_mat <- -1*lpm
  path <- shortest_path(d_eff_mat, source, target, node_costs = rep(0,nrow(d_eff_mat)))
  return(path)
}

#' Optimize All Probabilistic Path Values
#' 
#' Identify the path of optimal probabilistic path value from every source 
#' to every target in \code{sociomatrix}.
#' 
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param odds_scale a nonnegative real number indicating the observed tie
#'     strength value that corresponds to 1-1 transmission odds
#' @param odds_scale_by_node sets a transfer odds scale for each node in a 
#'     probabilistic path value calculation.  
#' 
#' @return All optimal paths from source to target nodes in \code{sociomatrix}. To
#'     minimize memory usage, paths are returned as a list of trees in Dijkstra's 
#'     format. Specific paths can be unpacked with \code{unpack} as described in the
#'     example below.
#' 
#' @seealso \code{\link{ppv}} to calculate the value of a user-specified path, 
#'     \code{\link{opt_ppv}} to identify the optimal path from a single source 
#'     node to a single target node
#' 
#' @example 
#' # Identify the optimal paths
#' best_paths <- all_opt_ppv(YangKnoke01, alpha = 1)
#' 
#' # 'best_paths' will contain a list of trees in Dijkstra's format.
#' # 'best_paths[[i]]' is the tree encoding shortest paths from source
#' #     node 'i' to all alters. We can return the optimal path from 
#' #     node 1 to node 4 as follows.
#' unpack(best_paths[[1]], source = 1, target = 4)
#' 
#' @export
all_opt_ppv <- function(sociomatrix, odds_scale = 1, odds_scale_by_node = NULL){
  check_input(sociomatrix, odds_scale = odds_scale, odds_scale_by_node = odds_scale_by_node)
  odds_matrix <- scale_to_odds(sociomatrix, odds_scale, odds_scale_by_node)
  probability_matrix <- odds_matrix/(1+odds_matrix)
  lpm <- log(probability_matrix)
  d_eff_mat <- -1*lpm
  paths <- APSP(d_eff_mat, node_costs = rep(0,nrow(d_eff_mat)))
  # list of spanning trees in dijkstra format
  return(paths)
}

context("Optimal paths")
library(gretel)


test_that("'opt_gpv' finds the optimal path", {
  # without and with node costs
  expect_equal(opt_gpv(BuchDarrah19, p = 1, source = 1, target = 5), c(1,3,5))
  expect_equal(opt_gpv(BuchDarrah19, p = 1, node_costs = c(0,0,5,0,0), source = 1, target = 5), c(1,2,4,5))
})

test_that("'all_opt_gpv' finds optimals paths", {
  # without and with node costs
  # expect_equal_to_reference(all_opt_gpv(BuchDarrah19, p = 0, node_costs = c(0,0,5,0,0)), file = "tests/testthat/ogpv_cache")
})

test_that("'opt_ppv' finds the optimal path", {
  # without and with nodewise odds scales
  expect_equal(opt_ppv(BuchDarrah19, source = 1, target = 5), c(1,3,5))
  expect_equal(opt_ppv(BuchDarrah19, odds_scale_by_node = c(1,1,9,1,1), source = 1, target = 5), c(1,5))
})

test_that("'all_opt_ppv' finds optimals paths", {
  # without and with nodewise odds scales
  # expect_equal_to_reference(all_opt_ppv(BuchDarrah19, odds_scale_by_node = c(1,1,9,1,1)), file = "tests/testthat/oppv_cache")
})

test_that("unpack returns a valid path", {
  expect_equal(unpack(c(2,4,5,NA,4), source = 4, target = 1), c(4,2,1))
})

test_that("all four optimal path functions fail gracefully", {
  # two checks each, one if bad input for path, one if bad input for specialty param
  expect_error(opt_gpv(BuchDarrah19, source = 1.5, target = 4))
  expect_error(all_opt_gpv(BuchDarrah19, p = -1))
  expect_error(opt_ppv(BuchDarrah19, source = "a", target = "b"))
  expect_error(all_opt_ppv(BuchDarrah19, odds_scale = 0))
})

test_that("unpack fails gracefully", {
  # target should be 4
  expect_equal(unpack(c(NA,4,5,NA,4), source = 2, target = 1), NA)
})

#' Example data from Yang, Knoke (2001)
#'
#' A sociomatrix encoding tie strengths among five nodes, used 
#' for examples in Yang, S., Knoke, D. (2001) Social Networks 23(4):285-295
#'
#' @format a numeric matrix with 5 rows and 5 columns
#' 
#' @source <DOI: 10.1016/S0378-8733(01)00043-0>
"YangKnoke01"

#' Example data from Opsahl, Agneessens, Skvoretz (2010)
#' 
#' A sociomatrix encoding tie strengths among five nodes, used for examples in
#' Opsahl, Agneessens, Skvoretz (2010) Social Networks 32(2010):245-251
#' 
#' @format a numeric matrix with 5 rows and 5 columns
#' 
#' @source  <DOI:10.1016/j.socnet.2010.03.006>
"OpsahlEtAl10"

#' Example data for gretel
#' 
#' A sociomatrix encoding tie strengths among five nodes
#' 
#' @format a numeric matrix with 5 rows and 5 columns
#' 
#' @source  <DOI:10.1016/j.socnet.2010.03.006>
"BuchDarrah19"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Find the shortest paths to other vertices
#' 
#' @param dist A matrix of distances between nodes
#' @param src An integer vertex ID 
#' @param node_costs a list of costs, in order, of all nodes represented in the 
#'     sociomatrix, all are assumed 0 if unspecified
#' @return A numeric vector, entry \emph{i} of which is the vertex immediately preceeding 
#'     vertex \emph{i} in the shortest path leading to \emph{i}. Full paths must be constructed
#'     recursively.
dijkstra_nodes <- function(dist, src, node_costs) {
    .Call(`_gretel_dijkstra_nodes`, dist, src, node_costs)
}

#' Find the shortest L-Inf norm paths to other vertices
#' 
#' @param dist A matrix of distances between nodes
#' @param src An integer vertex ID 
#' @return A numeric vector, entry \emph{i} of which is the vertex immediately preceeding 
#'   vertex \emph{i} in the shortest path leading to \emph{i}. Full paths must be constructed
#'   recursively.
dijkstra_inf <- function(dist, src) {
    .Call(`_gretel_dijkstra_inf`, dist, src)
}

#' Optimize Generalized Path Value
#' 
#' Identify the path of optimal generalized path value from a source node
#' to a target node.
#' 
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param source an integer index corresponding to a node in \code{sociomatrix}
#' @param target an integer index corresponding to a node in \code{sociomatrix}
#' @param p a nonnegative real number that sets the 'p-norm' parameter for 
#'     generalized path value calculation.
#' @param node_costs a list of costs, in order, of all nodes represented in the 
#'     sociomatrix, all are assumed 0 if unspecified
#'
#' @seealso \code{\link{gpv}} to calculate the value of a user-specified path,
#'     \code{\link{all_opt_gpv}} to simultaneously identify the optimal paths 
#'     from any source node to any target node.
#' 
#' @example
#' # Identify the optimal path
#' best_path <- opt_gpv(YangKnoke01, source = 1, target = 4, p = 1)
#' 
#' # Print the gpv of the optimal path
#' gpv(YangKnoke01, path = best_path, p = 1)
#' 
#' # Print the gpv of an inferior path
#' gpv(YangKnoke01, path = c(1,2,3,4))
#' 
#' @export
opt_gpv <- function(sociomatrix, source, target, p = Inf, node_costs = NULL){
  check_input(sociomatrix, source = source, target = target, p_norm = p, node_costs = node_costs)
  if(is.null(node_costs)){
    node_costs <- rep(0,nrow(sociomatrix))
  }
  if(p == 0){
    d_eff_mat <- 1/as_binary(sociomatrix)
    path <- shortest_path(d_eff_mat, source, target, node_costs)
  }else if(p < Inf){
    d_eff_mat <- 1/(sociomatrix^p)
    path <- shortest_path(d_eff_mat, source, target, node_costs)
  }else if(p == Inf){
    d_eff_mat <- 1/sociomatrix
    path <- shortest_path(d_eff_mat, source, target, node_costs = NULL, p_finite = F)
  }
  return(path)
}

#' Optimize All Generalized Path Values
#' 
#' Identify the path of optimal generalized path value from every source 
#' to every target in \code{sociomatrix}.
#' 
#' @param sociomatrix a nonnegative, real valued sociomatrix.
#' @param p a nonnegative real number that sets the 'p-norm' parameter for 
#'     generalized path value calculation.
#' @param node_costs a list of costs, in order, of all nodes represented in the 
#'     sociomatrix, all are assumed 0 if unspecified
#' 
#' @return All optimal paths from source to target nodes in \code{sociomatrix}. To
#'     minimize memory usage, paths are returned as a list of trees in Dijkstra's 
#'     format. Specific paths can be unpacked with \code{unpack} as described in the
#'     example below.
#' 
#' @seealso \code{\link{gpv}} to calculate the value of a user-specified path, 
#'     \code{\link{opt_gpv}} to identify the optimal path from a single source 
#'     node to a single target node
#' 
#' @example 
#' # Identify the optimal paths
#' best_paths <- all_opt_gpv(YangKnoke01, p = 1)
#' 
#' # 'best_paths' will contain a list of trees in Dijkstra's format.
#' # 'best_paths[[i]]' is the tree encoding shortest paths from source
#' #     node 'i' to all alters. We can return the optimal path from 
#' #     node 1 to node 4 as follows.
#' unpack(best_paths[[1]], source = 1, target = 4)
#' 
#' @export
all_opt_gpv <- function(sociomatrix, p = Inf, node_costs = NULL){
  check_input(sociomatrix, p_norm = p, node_costs = node_costs)
  if(is.null(node_costs)){
    node_costs <- rep(0,nrow(sociomatrix))
  }
  if(p == 0){
    d_eff_mat <- 1/as_binary(sociomatrix)
    paths <- APSP(d_eff_mat, node_costs)
  }else if(p < Inf){
    d_eff_mat <- 1/(sociomatrix^p)
    paths <- APSP(d_eff_mat, node_costs)
  }else if(p == Inf){
    d_eff_mat <- 1/sociomatrix
    paths <- APSP(d_eff_mat, node_costs = NULL, p_finite = F)
  }
  # list of spanning trees in dijkstra format
  return(paths)
}

## Third Submission
"""
Thanks,

Please always and only write package names, software names and API names 
in single quotes in the title and the description field.
f.i.: --> 'gretel'  &  --> ... generalized path value function ...

Please fix and resubmit.

Best,
Martina Schmirl
"""

Comments were addressed on 8-21-2019
* Fixed usage of single quotes in description field of DESCRIPTION

## Second submission
Reviewer Commments:
"""
Thanks,

The Description field is intended to be a (one paragraph) description
of what the package does and why it may be useful.
Please elaborate.

If there are references describing the methods in your package, please 
add these in the description field of your DESCRIPTION file in the form
authors (year) <doi:...>
authors (year) <arXiv:...>
authors (year, ISBN:...)
or if those are not available: authors (year) <https:...>
with no space after 'doi:', 'arXiv:', 'https:' and angle brackets for 
auto-linking.

Please always write TRUE and FALSE instead of T and F.

Best,
Martina Schmirl
""""

Comments were addressed on 08-20-2019
* Changed all 'T/F' logicals to 'TRUE/FALSE'
* Elaborated on description field  in DESCRIPTION
* Added references to description field in DESCRIPTION
* Re-ran R CMD check --as-cran with no new warnings, errors or notes

## Test environments
* local OS X install, R 3.6.1 and R-dev
* win-builder (devel and release)

## "R CMD check --as-cran" Results
There were no ERRORs or WARNINGs

There was 1 NOTE:

* checking CRAN incoming feasibility ... NOTE
Maintainer: ‘David Buch <davidbuch42@gmail.com>’
New submission

---
title: "Analyzing Networks with gretel"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Analyzing Networks with gretel}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

In analyzing social networks, it is often important to quantify the value of a path in terms of its constituent ties. We may be interested in identifying an optimal path (geodesic) for its own sake, or we may need to identify optimal paths to understand other structural properties such as betweenness centrality, which counts the number of optimal paths that rely on a certain node, or diameter of a network, which reports a 'distance' for the 'longest' (weakest) path in the network. Quotes are used around 'distance' because, except for the binary setting, there is no consensus as to how distance in a network ought to be calculated. *Yang and Knoke* (2001) offer three examples. *Opsahl,* et. al. (2010) offer a general framework containing many more.

Path value - intuitively, some inverted version of distance - shares this pathology, as do other structural properties. In fact, for any structural property of a network, there seem to be numerous proposals in the literature for their calculation. Sometimes, the competing approaches yield substantively different analytic conclusions, and may be published without guidance as to when one or the other calculation ought to be preferred.

Why do we need multiple measures? Is one patently right at the expense of the other? 

**gretel** was developed under the philosophy that a great deal of the discord in measures for certain structural properties can be reduced to a divergence in their approach to evaluating paths. Some measures implicitly describe paths by their weakest links, others by their number of edges, but all structural measures make a decision about how we should understand paths.

Therefore, the dimension of the problem can be greatly reduced. Instead of considering the pantheon of ways any structural property could be measured, we need only consider how we assign value to a path given tie strengths. We choose value over distance for the basis of our discussion because we see it as more fundamental - path values are reported in the same units as elemental tie strength data.

So now we are only left to decide whether there ought to be more than one way to appraise a path's value in terms of its ties. We believe that there should be. Different modeling assumptions about the ties suggest different methods for appraising paths. We believe that the importance of different modelling assumptions is underappreciated. Tie strength data have long been treated as one-dimensional. Granovetter (1973) acknowledged this is an oversimplification, but believed that the various dimensions (frequency, capacity, resource sharing, emotional depth, etc.) are sufficiently correlated that data about tie strength could be collected agnostically. While the facets of tie strength may be highly correlated, they should not necessarily be summarized the same way. For example, the maximum capacity of a path may be very well described by the capacity of the weakest tie, almost independent of its length. However, the resource sharing along a path will almost certainly be influenced by both the number of ties and the tie strengths. Further, the emotional depth of a path could be restricted significantly by the binary distance (number of intervening nodes). The upshot: both the dimension and situation must be considered in deciding how path value ought to be appraised.

**gretel** implements functions that quantify path values and identify optimal paths under a range of appraisal strategies. We extend ideas from *Opsahl*, et al (2010) and L-p spaces of mathematical analysis to develop path value measures parameterized by a real number p between 0 and Infinity. The values assigned to paths are comparable with tie strengths, and - for certain values of p - can be made to recapitulate the path appraisal strategies that underly all of the various structural measures we are aware of.

To illustrate the effect of the parameter p, we give three representative cases:

* p = 0 returns a path value based entirely on the number of edges in a path (relative strength of ties will not matter)

* p = 1 returns a path value that supposes strength is inversely proportional to cost and cost adds linearly. This is equivalent to the interpretation of a network as an electrical circuit and tie strengths as conductivity measures. Further analogies can be drawn to random walk probabilities (*Doyle, Snell* 2000)

* p = Infinity determines path value solely based on the weakest tie in the path. This is equivalent to Peay's 1980 path value measure. A physical analogy is fluid flow, in which bottlenecks place an upper limit on flow, dominating any effect of transport distance/resistance, etc.

Values of p between these represent compromises in the assumptions articulated above.

Here is an example of how different values of p, borne from different assumptions, can affect optimal path identification:

```{r setup}
library(gretel)
BuchDarrah19
```

```{r}
opt_gpv(BuchDarrah19, source = 1, target = 5, p = 0)
```

```{r}
opt_gpv(BuchDarrah19, source = 1, target = 5, p = 1)
```

```{r}
opt_gpv(BuchDarrah19, source = 1, target = 5, p = Inf)
```

We also introduce a new form of path value calculation, probabilistic path value, that extends the desirable probabilistic random walk interpretation of p = 1. This method requires the slightly stricter assumption that, up to a scale factor, tie strengths represent transmission odds (of some information, trend, or contagion) from one node to another, and the transmission events are independent. Using the same sociomatrix as above, suppose that a tie strength of 4 represents even transmission odds. Then we can find the path of optimal transmission 

```{r}
optimal_path <- opt_ppv(BuchDarrah19, source = 1, target = 5, odds_scale = 4)
print(optimal_path)
```

and calculate the transmission odds along the path.

```{r}
prob_path_value <- ppv(BuchDarrah19, path = optimal_path, odds_scale = 4)
transmission_odds <- prob_path_value/4
print(prob_path_value)
print(transmission_odds)
```

So if node 1 gets sick, the odds of node 5 getting sick via the optimal path are 1-4. The probability that node 5 will get sick along that path is 0.2.

Finally, the function **generate_proximities** returns a proximity matrix in one of three modes - generalized path value, probabilistic path value, and social conductivity. In the first two modes, the entries represent the values of the optimal paths between any pair of nodes. The third extends the conductivity model associated with p = 1 by evaluating the global conductivity between any two nodes. In all three cases, especially the third, the result is a psuedo-sociomatrix that captures richer, higher order interactions than would tie strengths alone.

```{r}
generate_proximities(BuchDarrah19, mode = "ogpv", p = 1)

generate_proximities(BuchDarrah19, mode = "oppv", odds_scale = 4)

generate_proximities(BuchDarrah19, mode = "sconductivity")
```

Below is an example of how **gretel** might be used in a betweenness centrality calculation:

```{r}
# Suppose we wish to calculate the betweenness centrality of node 3 in 
# the example sociomatrix 'BuchDarrah19'

# There are n-1 choose 2 pairs of nodes for which neither the source nor the target is 3
# Since n = 5 in this case, there are 6 pairs of nodes to consider.
# Therefore, node 3 mediates at most 6 shortest paths.
# (If this were a directed graph, this number would be 12)

all_paths <- all_opt_gpv(BuchDarrah19, p = 1)
paths_mediated <- 0
# We can get away with just looking at half the sociomatrix (below) because this
# one is undirected.
for(i in 1:5){
  for(j in (i+1:5)){
    shortest_ij <- unpack(all_paths[[i]], i, j)
    if(3 %in% shortest_ij) paths_mediated <- paths_mediated + 1
  }
}

# Print Betweenness Centrality
print(paths_mediated/6)


## What about if p = 0?
all_paths <- all_opt_gpv(BuchDarrah19, p = 0)
paths_mediated <- 0
# We can get away with just looking at half the sociomatrix (below) because this
# one is undirected.
for(i in 1:5){
  for(j in (i+1:5)){
    shortest_ij <- unpack(all_paths[[i]], i, j)
    if(3 %in% shortest_ij) paths_mediated <- paths_mediated + 1
  }
}

# Print Betweenness Centrality
print(paths_mediated/6)

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{BuchDarrah19}
\alias{BuchDarrah19}
\title{Example data for gretel}
\format{a numeric matrix with 5 rows and 5 columns}
\source{
<DOI:10.1016/j.socnet.2010.03.006>
}
\usage{
BuchDarrah19
}
\description{
A sociomatrix encoding tie strengths among five nodes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historical.R
\name{binary_distance}
\alias{binary_distance}
\title{Binary Distance of a Network Path}
\usage{
binary_distance(sociomatrix, path)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{path}{an integer vector of node indices from \code{sociomatrix}.}
}
\description{
Calculates the binary distance of a user-specified network path through a network,
if all edges exist. Otherwise, returns \code{Inf} to signify infinite distance.
}
\examples{
## Calculate binary distance along a path in a sociomatrix
binary_distance(YangKnoke01, path = c(1,2,5))

## This path doesn't exist
binary_distance(YangKnoke01, path = c(1,2,4,5))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historical.R
\name{peay_average_path_value}
\alias{peay_average_path_value}
\title{Yang and Knoke's Average Path Value}
\usage{
peay_average_path_value(sociomatrix, path)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{path}{an integer vector of node indices from \code{sociomatrix}.}
}
\description{
Calculates 'APV' (Average Path Value) as defined in Yang, Knoke (2001)
Called \code{peay_average_path_value} in homage to E.R. Peay, who defined 
path length in 1980.
}
\examples{
## Calculate 'APV' of a path in a sociomatrix
peay_average_path_value(YangKnoke01, path = c(1,2,5))

## This path doesn't exist
peay_average_path_value(YangKnoke01, path = c(1,2,4,5))

}
\seealso{
\code{\link{peay_path_value}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/support.R
\name{unpack}
\alias{unpack}
\title{Unpacks a Path from a Dijkstra-Format Spanning Tree}
\usage{
unpack(tree, source, target)
}
\arguments{
\item{tree}{a Dijkstra-format tree returned by \code{all_opt_gpv} or \code{all_opt_ppv}}

\item{source}{an integer index corresponding to a node in \code{sociomatrix}}

\item{target}{an integer index corresponding to a node in \code{sociomatrix}}
}
\description{
Used with \code{all_opt_gpv} and \code{all_opt_ppv} to 
unpack individual paths from the Dijkstra-format trees that 
those functions return.
}
\details{
Returns \code{NA} if a path does not exist
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opt_ppv.R
\name{opt_ppv}
\alias{opt_ppv}
\title{Optimize Probabilistic Path Value}
\usage{
opt_ppv(sociomatrix, source, target, odds_scale = 1,
  odds_scale_by_node = NULL)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{source}{an integer index corresponding to a node in \code{sociomatrix}}

\item{target}{an integer index corresponding to a node in \code{sociomatrix}}

\item{odds_scale}{a nonnegative real number indicating the observed tie
strength value that corresponds to 1-1 transmission odds}

\item{odds_scale_by_node}{sets a transfer odds scale for each node in a 
probabilistic path value calculation.}
}
\description{
Identify the path of optimal probabilistic path value from a source node
to a target node.
}
\seealso{
\code{\link{ppv}} to calculate the value of a user-specified path,
    \code{\link{all_opt_ppv}} to simultaneously identify the optimal paths 
    from any source node to any target node.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proximity.R
\name{generate_proximities}
\alias{generate_proximities}
\title{Generate a Proximity Matrix}
\usage{
generate_proximities(sociomatrix, mode = c("ogpv", "oppv",
  "sconductivity"), p = Inf, node_costs = NULL, odds_scale = 1,
  odds_scale_by_node = NULL)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{mode}{a selection of \code{'ogpv'}, \code{'oppv'}, or \code{'sconductivity'}}

\item{p}{if \code{mode} is \code{'ogpv'}, determines 'p-norm' parameter for 
generalized path value calculation.}

\item{node_costs}{if \code{mode} is \code{'ogpv'}, assigns transmission costs to
vertices within the sociomatrix.}

\item{odds_scale}{if \code{mode} is \code{'oppv'}, sets a global transfer odds 
scale for probabilistic path value calculation.}

\item{odds_scale_by_node}{if \code{mode} is \code{'oppv'}, sets a transfer odds 
scale for each node in a probabilistic path value calculation.}
}
\description{
Generates a proximity matrix in one of three modes:
\describe{
  \item{\code{'ogpv'}}{Optimal Generalized Path Value. Entry \code{i,j} of the 
    proximity matrix will equal the optimal 'gpv' among all paths connecting node 
    \code{i} to node \code{j}.}
  \item{\code{'oppv'}}{Optimal Probabilistic Path Value. Entry \code{i,j} of the 
    proximity matrix will equal the optimal 'ppv' among all paths connecting node 
    \code{i} to node \code{j}.}
  \item{\code{'sconductivity'}}{Social Conductivity (Random Walk Probability). If 
    each tie strength recorded in \code{sociomatrix} is taken to be analogous to the 
    conductivity of an electrical component, \code{i,j} of the proximity matrix 
    will equal total conductivity of all paths from node \code{i} to node \code{j}.}
}
}
\examples{
## Generate a proximity matrix in each mode
## Optimal Generalized Path Value
generate_proximities(YangKnoke01, mode = "ogpv", p = Inf, node_costs = c(1,3,3,2,1))

## Optimal Probabilistic Path Value
generate_proximities(YangKnoke01, mode = "oppv", odds_scale = 2)

## Sconductivity
generate_proximities(YangKnoke01, mode = "sconductivity")

}
\seealso{
\code{\link{gpv}}, \code{\link{ppv}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opt_ppv.R
\name{all_opt_ppv}
\alias{all_opt_ppv}
\title{Optimize All Probabilistic Path Values}
\usage{
all_opt_ppv(sociomatrix, odds_scale = 1, odds_scale_by_node = NULL)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{odds_scale}{a nonnegative real number indicating the observed tie
strength value that corresponds to 1-1 transmission odds}

\item{odds_scale_by_node}{sets a transfer odds scale for each node in a 
probabilistic path value calculation.}
}
\value{
All optimal paths from source to target nodes in \code{sociomatrix}. To
    minimize memory usage, paths are returned as a list of trees in Dijkstra's 
    format. Specific paths can be unpacked with \code{unpack} as described in the
    example below.
}
\description{
Identify the path of optimal probabilistic path value from every source 
to every target in \code{sociomatrix}.
}
\seealso{
\code{\link{ppv}} to calculate the value of a user-specified path, 
    \code{\link{opt_ppv}} to identify the optimal path from a single source 
    node to a single target node
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{YangKnoke01}
\alias{YangKnoke01}
\title{Example data from Yang, Knoke (2001)}
\format{a numeric matrix with 5 rows and 5 columns}
\source{
<DOI: 10.1016/S0378-8733(01)00043-0>
}
\usage{
YangKnoke01
}
\description{
A sociomatrix encoding tie strengths among five nodes, used 
for examples in Yang, S., Knoke, D. (2001) Social Networks 23(4):285-295
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppv.R
\name{ppv}
\alias{ppv}
\title{Calculate probabilistic path value}
\usage{
ppv(sociomatrix, path, odds_scale = 1, odds_scale_by_node = NULL)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{path}{an integer vector of node indices from \code{sociomatrix}.}

\item{odds_scale}{a nonnegative real number indicating the observed tie
strength value that corresponds to 1-1 transmission odds}

\item{odds_scale_by_node}{sets a transfer odds scale for each node in a 
probabilistic path value calculation.}
}
\description{
Given a real valued sociomatrix, a path, and an optional \code{odds_scale}, \code{ppv}
calculates the transmission odds for the path and returns the transmission odds
times \code{odds_scale} so the result can be directly compared with observed 
tie strenghts.
}
\details{
We assume that observed tie strengths in \code{sociomatrix} are linearly
proportional to transmission odds. That is, if the transmission odds for
a strength 1 tie are 1 to 1, the transmission odds for a strength 5 tie are
1 to 5.
}
\examples{
## Calculate ppv along a path in a sociomatrix
ppv(YangKnoke01, path = c(1,2,5), odds_scale = 3)

## This path doesn't exist
gpv(YangKnoke01, path = c(1,2,4,5))

}
\seealso{
\code{\link{opt_ppv}} to identify the path of optimal 'ppv' between two nodes 
    and \code{\link{all_opt_ppv}} to identify the optimal paths between all pairs of 
    nodes. Calling \code{\link{generate_proximities}} with \code{mode = 'ppv'} 
    returns a matrix 'ppv' values for the optimal paths between all pairs of 
    nodes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opt_gpv.R
\name{all_opt_gpv}
\alias{all_opt_gpv}
\title{Optimize All Generalized Path Values}
\usage{
all_opt_gpv(sociomatrix, p = Inf, node_costs = NULL)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{p}{a nonnegative real number that sets the 'p-norm' parameter for 
generalized path value calculation.}

\item{node_costs}{a list of costs, in order, of all nodes represented in the 
sociomatrix, all are assumed 0 if unspecified}
}
\value{
All optimal paths from source to target nodes in \code{sociomatrix}. To
    minimize memory usage, paths are returned as a list of trees in Dijkstra's 
    format. Specific paths can be unpacked with \code{unpack} as described in the
    example below.
}
\description{
Identify the path of optimal generalized path value from every source 
to every target in \code{sociomatrix}.
}
\seealso{
\code{\link{gpv}} to calculate the value of a user-specified path, 
    \code{\link{opt_gpv}} to identify the optimal path from a single source 
    node to a single target node
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opt_gpv.R
\name{opt_gpv}
\alias{opt_gpv}
\title{Optimize Generalized Path Value}
\usage{
opt_gpv(sociomatrix, source, target, p = Inf, node_costs = NULL)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{source}{an integer index corresponding to a node in \code{sociomatrix}}

\item{target}{an integer index corresponding to a node in \code{sociomatrix}}

\item{p}{a nonnegative real number that sets the 'p-norm' parameter for 
generalized path value calculation.}

\item{node_costs}{a list of costs, in order, of all nodes represented in the 
sociomatrix, all are assumed 0 if unspecified}
}
\description{
Identify the path of optimal generalized path value from a source node
to a target node.
}
\seealso{
\code{\link{gpv}} to calculate the value of a user-specified path,
    \code{\link{all_opt_gpv}} to simultaneously identify the optimal paths 
    from any source node to any target node.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{dijkstra_nodes}
\alias{dijkstra_nodes}
\title{Find the shortest paths to other vertices}
\usage{
dijkstra_nodes(dist, src, node_costs)
}
\arguments{
\item{dist}{A matrix of distances between nodes}

\item{src}{An integer vertex ID}

\item{node_costs}{a list of costs, in order, of all nodes represented in the 
sociomatrix, all are assumed 0 if unspecified}
}
\value{
A numeric vector, entry \emph{i} of which is the vertex immediately preceeding 
    vertex \emph{i} in the shortest path leading to \emph{i}. Full paths must be constructed
    recursively.
}
\description{
Find the shortest paths to other vertices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historical.R
\name{flament_average_path_length}
\alias{flament_average_path_length}
\title{Yang and Knoke's Average Path Length}
\usage{
flament_average_path_length(sociomatrix, path)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{path}{an integer vector of node indices from \code{sociomatrix}.}
}
\description{
Calculates 'APL' (Average Path Length) as defined in Yang, Knoke (2001).
Called \code{flament_average_path_length} in homage to A.C. Flament, who defined
path length in 1963.
}
\examples{
## Calculate 'APL' of a path in a sociomatrix
flament_average_path_length(YangKnoke01, path = c(1,2,5))

## This path doesn't exist
flament_average_path_length(YangKnoke01, path = c(1,2,4,5))

}
\seealso{
\code{\link{flament_path_length}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historical.R
\name{peay_path_value}
\alias{peay_path_value}
\title{Peay's Path Value Measure}
\usage{
peay_path_value(sociomatrix, path)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{path}{an integer vector of node indices from \code{sociomatrix}.}
}
\description{
Calculates path value as defined in Peay (1980). That is, returns the
value of the weakest connection in the path, if all edges exist. 
Otherwise, returns 0.
}
\examples{
## Calculate Peay's Path Value along a path in a sociomatrix
peay_path_value(YangKnoke01, path = c(1,2,5))

## This path doesn't exist
peay_path_value(YangKnoke01, path = c(1,2,4,5))

}
\seealso{
\code{\link{peay_average_path_value}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historical.R
\name{flament_path_length}
\alias{flament_path_length}
\title{Flament's Path Length Measure}
\usage{
flament_path_length(sociomatrix, path)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{path}{an integer vector of node indices from \code{sociomatrix}.}
}
\description{
Calculates path length as defined in Flament (1963). That is, sums the
values of each edge in the path, if all edges exist. Otherwise, returns \code{NA}.
}
\examples{
## Calculate Flament's Path Length along a path in a sociomatrix
flament_path_length(YangKnoke01, path = c(1,2,5))

## This path doesn't exist
flament_path_length(YangKnoke01, path = c(1,2,4,5))

}
\seealso{
\code{\link{flament_average_path_length}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{dijkstra_inf}
\alias{dijkstra_inf}
\title{Find the shortest L-Inf norm paths to other vertices}
\usage{
dijkstra_inf(dist, src)
}
\arguments{
\item{dist}{A matrix of distances between nodes}

\item{src}{An integer vertex ID}
}
\value{
A numeric vector, entry \emph{i} of which is the vertex immediately preceeding 
  vertex \emph{i} in the shortest path leading to \emph{i}. Full paths must be constructed
  recursively.
}
\description{
Find the shortest L-Inf norm paths to other vertices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpv.R
\name{gpv}
\alias{gpv}
\title{Generalized Path Value}
\usage{
gpv(sociomatrix, path, p = Inf, node_costs = NULL)
}
\arguments{
\item{sociomatrix}{a nonnegative, real valued sociomatrix.}

\item{path}{an integer vector of node indices from \code{sociomatrix}.}

\item{p}{a nonnegative real number that sets the 'p-norm' parameter for 
generalized path value calculation.}

\item{node_costs}{a list of costs, in order, of all nodes represented in the 
sociomatrix, all are assumed 0 if unspecified}
}
\description{
Calculates the generalized path value of a user-specified path through 
\code{sociomatrix}. Parameter \code{p} sets the p-norm used in calculation.
}
\details{
As a rule of thumb, p close to 0 will downweight the impact of particular
tie strengths and upweight the impact of binary path length. p equal to 
infinity will recapitulate the traditional path value measure of Peay (1980)
and is therefore the default. In other words, the value of a path under 
\code{p = Inf} will be the value of the weakest tie. The value of the same
path under \code{p = 0} will be the inverse of its binary length.
}
\examples{
## Calculate gpv along a path in a sociomatrix
gpv(YangKnoke01, path = c(1,2,5), p = 1)

## The same calculation, with nonzero node costs
gpv(YangKnoke01, path = c(1,2,5), p = 1, node_costs = c(1,3,3,2,1))

## This path doesn't exist
gpv(YangKnoke01, path = c(1,2,4,5), p = 0)

}
\seealso{
\code{\link{opt_gpv}} to identify the path of optimal 'gpv' between two nodes 
    and \code{\link{all_opt_gpv}} to identify the optimal paths between all pairs of 
    nodes. Calling \code{\link{generate_proximities}} with \code{mode = 'gpv'} 
    returns a matrix 'gpv' values for the optimal paths between all pairs of 
    nodes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gretel.R
\docType{package}
\name{gretel}
\alias{gretel}
\alias{gretel-package}
\title{sconduct: Generalized Path Analysis for Social Networks}
\description{
This package contains two categories of functions. The first category is concerned
with assigning values to user specified paths, while the second identifies
paths of optimal value.
}
\details{
Key functions in the path value calculation category are
- \code{gpv}, which calculates Generalized Path Value
- \code{ppv}, which calculates Probabilistic Path Value
- \code{binary_distance}, \code{peay_path_value}, \code{flament_path_length},
  \code{peay_average_path_value}, and \code{flament_average_path_length}, which
  calculate path value measures described in \emph{Yang, Knoke} (2001).
- \code{generate_proximities}, which generates a matrix of values representing the
  measures of optimal paths from each source node (row index) to each target node 
  (column index).
  
Key functions in the optimal path identification category are
- \code{opt_gpv}, which identifies the path of optimal Generalized Path Value from 
  a particular source node to a particular target node
- \code{opt_ppv}, which identifies the path of optimal Probabilistic Path Value from
  a particular source node to a particular target node
- \code{all_opt_gpv}, which identifies the 'gpv'-optimal paths from every source node 
  to every target node
- \code{all_opt_ppv}, which identifies the 'ppv'-optimal paths from every source node
  to every target node
- \code{unpack}, which unpacks the Dijkstra-format encoded shortest paths returned by
  \code{all_opt_gpv} and \code{all_opt_ppv}. See their help pages for details.
}
